FuriousUI = { }
FuriousUIState = { }

--
--
-- locals
local dewdrop = AceLibrary:GetInstance('Dewdrop-2.0')

local comm = ChatThrottleLib

local timertostring = function (s)
	if not s then return 'off' end
	return string.format('%02d:%02d', math.floor(s/60), math.fmod(s, 60))
end

local tablecount = function (_t)
	local count = 0
	for k, v in pairs(_t) do count = count + 1 end
	return count
end

--
--
-- comparators
local comparealpha = function (a, b)
	return a < b
end
local comparealphainverse = function (a, b) return comparealpha(b, a) end

local compareplayerclass = function (a, b)
	local pa = FuriousRaidDB.players[a] or 'unknown'
	local pb = FuriousRaidDB.players[b] or 'unknown'

	if pa.class == pb.class then return a < b end
	return pa.class < pb.class
end
local compareplayerclassinverse = function (a, b) return compareplayerclass(b, a) end

local compareplayervalue = function (a, b)
	local va = FuriousRaid.ExecuteCommand('playerscore', FuriousRaid.state.selected.set, a)
	local vb = FuriousRaid.ExecuteCommand('playerscore', FuriousRaid.state.selected.set, b)

	if va == vb then return a < b end
	return va < vb
end
local compareplayervalueinverse = function (a, b) return compareplayervalue(b, a) end

FuriousUIState.listcomparator = comparealpha

--
--
-- basic display
FuriousUI.Show = function ()
	FuriousUI.UpdateSet()
	FuriousUI.UpdateList()
	FuriousUI.UpdateSetDetail()
	FuriousUI.UpdatePlayerDetail()
	FuriousUI.UpdateWaitlist()

	FuriousUIFrames.frames.main:Show()
end

FuriousUI.Hide = function ()
	FuriousUI.ClearSet()
	FuriousUI.ClearList()
	FuriousUI.ClearSetDetail()
	FuriousUI.ClearPlayerDetail()

	FuriousUIFrames.frames.main:Hide()
end

FuriousUI.ClearSet = function ()
	for k, v in ipairs(FuriousUIFrames.groups.set.groups.buttons.frames) do
		v:Hide()
	end
end

FuriousUI.ClearList = function ()
	for k, v in ipairs(FuriousUIFrames.groups.list.groups.buttons.frames) do
		v:Hide()
	end
end

FuriousUI.ClearPlayerDetail = function ()
	for k, v in ipairs(FuriousUIFrames.groups.playerdetail.groups.buttons.frames) do
		v:Hide()
	end
end

FuriousUI.ClearSetDetail = function ()
	for k, v in ipairs(FuriousUIFrames.groups.setdetail.groups.buttons.frames) do
		v:Hide()
	end
end

FuriousUI.UpdateSet = function ()
	local p = { }
	for k, v in pairs(FuriousRaidDB.sets) do table.insert(p, k) end
	table.sort(p)


	FauxScrollFrame_Update(FUISetScroll, #p, #(FuriousUIFrames.groups.set.groups.buttons.frames), 18)
	local offset = FauxScrollFrame_GetOffset(FUISetScroll)

	for i=1, #(FuriousUIFrames.groups.set.groups.buttons.frames), 1 do		
		local f = FuriousUIFrames.groups.set.groups.buttons.frames[i]

		if (i + offset) <= #p then
			local t = FuriousRaidDB.sets[p[i + offset]].config.timer.remaining
			f:SetMultiText(p[i + offset], FuriousRaidDB.sets[p[i + offset]].caste, timertostring(t))

			if p[i + offset] == FuriousRaid.state.selected.set then
				f:SetBackdropColor(0.5, 0.5, 0.9, 0.7)
			else
				if (i + offset) % 2 == 0 then
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.0)
				else
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.25)
				end
			end

			f:Show()
		else
			f:Hide()
		end
	end
end

FuriousUI.UpdateSetTimers = function ()
	for k, v in ipairs(FuriousUIFrames.groups.set.groups.buttons.frames) do
		if v:IsVisible() then
			local t = FuriousRaidDB.sets[v.fTextA:GetText()].config.timer.remaining
			v.fTextC:SetText(timertostring(t))
		end
	end
end

FuriousUI.UpdateList = function ()
	if not FuriousRaid.state.selected.set then return end

	local p = { }
	for k, v in pairs(FuriousRaidDB.players) do
		if FuriousRaid.IsFiltered(k) then
			table.insert(p, k)
		end
	end
	table.sort(p, FuriousUIState.listcomparator)

	FauxScrollFrame_Update(FUIListScroll, #p, #(FuriousUIFrames.groups.list.groups.buttons.frames), 18)
	local offset = FauxScrollFrame_GetOffset(FUIListScroll)

	for i=1, #(FuriousUIFrames.groups.list.groups.buttons.frames), 1 do
		local f = FuriousUIFrames.groups.list.groups.buttons.frames[i]

		if (i + offset) <= #p then
			local class = FuriousRaidDB.players[p[i + offset]].class or 'unknown'
			local value = FuriousRaid.ExecuteCommand('playervaluestring', FuriousRaid.state.selected.set, p[i + offset])

			f:SetMultiText(p[i + offset], string.lower(class), value )
			f.fTextB:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b)


			if p[i + offset] == FuriousRaid.state.selected.player then
				f:SetBackdropColor(0.5, 0.5, 0.7, 0.9)
			else
				if (i + offset) % 2 == 0 then
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.0)
				else
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.25)
				end
			end

			f:Show()
		else
			f:Hide()
		end
	end
end

FuriousUI.UpdatePlayerDetail = function ()
	if not FuriousRaid.state.selected.player then return end

	FuriousUIFrames.groups.playerdetail.groups.buttons.frames.header:SetMultiText(FuriousRaid.state.selected.player)

	local p = { }
	for k, v in pairs(FuriousRaidDB.players[FuriousRaid.state.selected.player].items) do
		table.insert(p, k)
	end
	table.sort(p, function(a, b) return a > b end)

	FauxScrollFrame_Update(FUIPlayerDetailScroll, #p, #(FuriousUIFrames.groups.playerdetail.groups.buttons.frames), 54)
	local offset = FauxScrollFrame_GetOffset(FUIPlayerDetailScroll)

	for i=1, #(FuriousUIFrames.groups.playerdetail.groups.buttons.frames), 1 do
		local f = FuriousUIFrames.groups.playerdetail.groups.buttons.frames[i]

		if (i + offset) <= #p then
			f:SetItem(FuriousRaidDB.players[FuriousRaid.state.selected.player].items[p[i + offset]], p[i + offset])

			if tostring(p[i + offset]) == tostring(FuriousRaid.state.selected.detail) then
				f:SetBackdropColor(0.5, 0.5, 0.7, 0.9)
			else
				if (i + offset) % 2 == 0 then
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.0)
				else
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.25)
				end
			end

			f:Show()
		else
			f:Hide()
		end
	end
end

FuriousUI.UpdateSetDetail = function ()
	if not FuriousRaid.state.selected.set then return end

	FuriousUIFrames.groups.setdetail.groups.buttons.frames.header:SetMultiText(FuriousRaid.state.selected.set)

	local p = { }
	for k, v in pairs(FuriousRaidDB.sets[FuriousRaid.state.selected.set].monitors) do
		table.insert(p, k)
	end
	table.sort(p, function(a, b) return a > b end)

	FauxScrollFrame_Update(FUISetDetailScroll, #p, #(FuriousUIFrames.groups.setdetail.groups.buttons.frames), 54)
	local offset = FauxScrollFrame_GetOffset(FUISetDetailScroll)

	for i=1, #(FuriousUIFrames.groups.setdetail.groups.buttons.frames), 1 do
		local f = FuriousUIFrames.groups.setdetail.groups.buttons.frames[i]

		if (i + offset) <= #p then
			f:SetItem(FuriousRaidDB.sets[FuriousRaid.state.selected.set].monitors[p[i + offset]], p[i + offset])

			if tostring(p[i + offset]) == tostring(FuriousRaid.state.selected.detail) then
				f:SetBackdropColor(0.5, 0.5, 0.7, 0.9)
			else
				if (i + offset) % 2 == 0 then
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.0)
				else
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.25)
				end
			end

			f:Show()
		else
			f:Hide()
		end
	end
end

FuriousUI.UpdateWaitlist = function ()
	local p = { }
	for k, v in pairs(FuriousRaidDB.waitlist) do
		table.insert(p, k)
	end
	table.sort(p)

	FauxScrollFrame_Update(FUIWaitlistScroll, #p, #(FuriousUIFrames.groups.waitlist.groups.buttons.frames), 18)
	local offset = FauxScrollFrame_GetOffset(FUIWaitlistScroll)

	for i=1, #(FuriousUIFrames.groups.waitlist.groups.buttons.frames), 1 do
		local f = FuriousUIFrames.groups.waitlist.groups.buttons.frames[i]

		if (i + offset) <= #p then
			local class = 'unknown'
			if FuriousRaidDB.players[p[i + offset]] then
				class = FuriousRaidDB.players[p[i + offset]].class
				f.fTextB:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b)
			else
				f.fTextB:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			end

			f:SetMultiText(p[i + offset], string.lower(class))

			if p[i + offset] == FuriousRaid.state.selected.waitlist then
				f:SetBackdropColor(0.5, 0.5, 0.7, 0.9)
			else
				if (i + offset) % 2 == 0 then
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.0)
				else
					f:SetBackdropColor(0.5, 0.5, 0.7, 0.25)
				end
			end

			f:Show()
		else
			f:Hide()
		end
	end
end



--
--
-- Button clicks
local furiousUIL = { }

furiousUIL.onClickMonitorButton = function (f)
	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	if FuriousRaid.state.monitor.stateid == 'STOPPED' then
		FuriousRaid.ResetMonitorState()

		local lsi
		local lf = LootFrame
		if XLootFrame then lf = XLootFrame end

		if lf:IsVisible() then
			for lsi = 1, GetNumLootItems() do
				looticon, lootname, lootquantity, lootrarity = GetLootSlotInfo(lsi)

				if lootrarity >= 3 then
					lootlink = GetLootSlotLink(lsi)
					_, _, lootid = string.find(lootlink, 'item:([0-9]+):')

					FuriousRaid.state.monitor.item = {
						name = lootname,
						id = tonumber(lootid),
						link = lootlink,
					}
					if FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.autofilter then
						FuriousRaid.SetFilterFromItem(true, lootlink)
						FuriousUIFrames.frames.filtereditbox:SetText(FuriousRaid.GetFilterString())
					end

					break
				end
			end
		elseif FuriousRaid.state.whisperloot then
			lootname, lootlink = GetItemInfo(FuriousRaid.state.whisperloot)

			FuriousRaid.state.monitor.whisperloot = true
			FuriousRaid.state.monitor.item = {
				name = lootname,
				id = FuriousRaid.state.whisperloot,
				link = lootlink,
			}
			FuriousRaid.state.whisperloot = nil

			if FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.autofilter then
				FuriousRaid.SetFilterFromItem(true, lootlink)
				FuriousUIFrames.frames.filtereditbox:SetText(FuriousRaid.GetFilterString())
			end
		else
			print(FuriousRaid.prefixprint, 'ERROR -- no loot to use')
			return
		end

		FuriousUI.UpdateList()
	end

	FuriousRaid.ExecuteCommand('monitor', FuriousRaid.state.monitor.set, FuriousRaid.state.monitor)
	f:SetText(FuriousRaid.state.monitor.buttontext)
end

furiousUIL.onClickClearWaitlistButton = function (f)
	FuriousRaidDB.waitlist = { }

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateWaitlist()
end

furiousUIL.onClickHeaderSetButton = function (f)
	FuriousUIFrames.frames.filtereditbox:ClearFocus()
end

furiousUIL.onClickNormalSetButton = function (f, b)
	FuriousRaid.state.selected.set = f.fTextA:GetText()
	FuriousRaid.state.selected.player = nil
	FuriousRaid.state.selected.detail = nil

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateSet()
	FuriousUI.UpdateList()
	FuriousUI.UpdateSetDetail()

	FuriousUI.ClearPlayerDetail()


	dewdrop:Close()
	if b == 'LeftButton' then
	else
		dewdrop:Open(f, 'children', furiousUIL.menuNormalSetButton)
	end
end

furiousUIL.onClickHeaderListButton = function (f)
	FuriousUIState.listcomparator = f:fComparator()
	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateList()
end

furiousUIL.onClickNormalListButton = function (f, b)
	FuriousRaid.state.selected.player = f.fTextA:GetText()
	FuriousRaid.state.selected.detail = nil

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateList()	
	FuriousUI.UpdateSetDetail()
	FuriousUI.UpdatePlayerDetail()

	dewdrop:Close()
	if b == 'LeftButton' then
	else
		dewdrop:Open(f, 'children', furiousUIL.menuNormalListButton)
	end
end

furiousUIL.onClickHeaderWaitlistButton = function (f)
	FuriousUIFrames.frames.filtereditbox:ClearFocus()
end

furiousUIL.onClickNormalWaitlistButton = function (f, b)
	FuriousRaid.state.selected.waitlist = f.fTextA:GetText()

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateWaitlist()

	dewdrop:Close()
	if b == 'LeftButton' then
	else
		dewdrop:Open(f, 'children', furiousUIL.menuNormalWaitlistButton)
	end
end

furiousUIL.onClickNormalPlayerDetailButton = function (f)
	FuriousRaid.state.selected.detail = f.fID

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdatePlayerDetail()

	local t
	if ChatFrameEditBox:IsVisible() then
		if IsShiftKeyDown() then
			t = ChatFrameEditBox:GetText() .. f.fLink
			ChatFrameEditBox:SetText(t)
		elseif IsAltKeyDown() then
			t = ChatFrameEditBox:GetText() .. f.fTextA:GetText() .. ': ' .. f.fLink .. ' in ' .. f.fTextB:GetText()
			ChatFrameEditBox:SetText(t)
		end
	end
end

furiousUIL.onClickNormalSetDetailButton = function (f)
	FuriousRaid.state.selected.detail = f.fID

	FuriousUIFrames.frames.filtereditbox:ClearFocus()

	FuriousUI.UpdateSetDetail()

	local t
	if ChatFrameEditBox:IsVisible() then
		if IsShiftKeyDown() then
			t = ChatFrameEditBox:GetText() .. f.fLink
			ChatFrameEditBox:SetText(t)
		elseif IsAltKeyDown() then
			t = ChatFrameEditBox:GetText() .. f.fTextA:GetText() .. ': ' .. f.fLink .. ', ' .. f.fTextB:GetText()
			ChatFrameEditBox:SetText(t)
		end
	end
end

furiousUIL.onClickFRaidButton = function (f, b)
	dewdrop:Close()
	if b == 'RightButton' then
		dewdrop:Open(f, 'children', furiousUIL.menuFRaidButton)
	end
end

--
furiousUIL.menuNormalListButton = function (_level ,_value)
	local s = FuriousRaid.state.selected.set
	local p = FuriousRaid.state.selected.player

	dewdrop:AddLine(
		'text', 'Player: |cffaaffaa' .. (p or "unknown") .. '|r',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Add',
		'hasArrow', true,
		'hasEditBox', true,
		'editBoxText', '',
		'editBoxFunc', function(a) FuriousRaid.ExecuteCommand('add', s, p, tonumber(a)) FuriousUI.UpdateList() end,
		'editBoxValidateFunc', function(a) return (tonumber(a) ~= nil) end
	)
	dewdrop:AddLine(
		'text', 'Clear',
		'func', function() FuriousRaid.ExecuteCommand('clear', s, p) FuriousUI.UpdateList() end
	)

	if FuriousRaid.state.monitor.enableaward and FuriousRaid.state.monitor.item and FuriousRaid.state.monitor.item.link then
		dewdrop:AddLine(
			'text',	'',
			'notClickable', true
		)
		dewdrop:AddLine(
			'text', 'Award: ' .. FuriousRaid.state.monitor.item.link,
			'func', function()
				if FuriousRaid.MasterLoot(p, FuriousRaid.state.monitor.item.id) then
					FuriousRaid.ExecuteCommand('itemaward', FuriousRaid.state.monitor.set, p, FuriousRaid.state.monitor)
					FuriousUIFrames.frames.monitor:SetText(FuriousRaid.state.monitor.buttontext)
					FuriousUI.UpdateList()
					FuriousUI.UpdateSetDetail()
					FuriousUI.UpdatePlayerDetail()
				elseif FuriousRaid.state.monitor.whisperloot then
					print(FuriousRaid.prefixprint, 'Whisperloot -- item recorded, but not masterlooted')
					FuriousRaid.ExecuteCommand('itemaward', FuriousRaid.state.monitor.set, p, FuriousRaid.state.monitor)
					FuriousUIFrames.frames.monitor:SetText(FuriousRaid.state.monitor.buttontext)
					FuriousUI.UpdateList()
					FuriousUI.UpdateSetDetail()
					FuriousUI.UpdatePlayerDetail()
				else
					print(FuriousRaid.prefixprint, 'Error awarding item -- item not masterlooted')
				end

				dewdrop:Close()
			end
		)
		dewdrop:AddLine(
			'text',	'',
			'notClickable', true
		)
		dewdrop:AddLine(
			'text', 'Disenchant/Bank: ' .. FuriousRaid.state.monitor.item.link,
			'func', function()
				if FuriousRaid.MasterLoot(p, FuriousRaid.state.monitor.item.id) then
					FuriousRaid.ExecuteCommand('itembank', FuriousRaid.state.monitor.set, p, FuriousRaid.state.monitor)
					FuriousUIFrames.frames.monitor:SetText(FuriousRaid.state.monitor.buttontext)
					FuriousUI.UpdateList()
					FuriousUI.UpdateSetDetail()
					FuriousUI.UpdatePlayerDetail()
				elseif FuriousRaid.state.monitor.whisperloot then
					print(FuriousRaid.prefixprint, 'Whisperloot -- item recorded, but not masterlooted')
					FuriousRaid.ExecuteCommand('itembank', FuriousRaid.state.monitor.set, p, FuriousRaid.state.monitor)
					FuriousUIFrames.frames.monitor:SetText(FuriousRaid.state.monitor.buttontext)
					FuriousUI.UpdateList()
					FuriousUI.UpdateSetDetail()
					FuriousUI.UpdatePlayerDetail()
				else
					print(FuriousRaid.prefixprint, 'Error banking item -- item not masterlooted')
				end

				dewdrop:Close()
			end
		)
	end

	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
 	dewdrop:AddLine(
		'text', 'Remove!',
		'func', function()
			FuriousRaid.RemovePlayer(p)
			FuriousRaid.state.selected.player = nil

			FuriousUI.UpdateList()
			FuriousUI.ClearPlayerDetail()

			dewdrop:Close()
		end
	)
end

furiousUIL.menuNormalSetButton = function (_level, _value)
	local s = FuriousRaid.state.selected.set

	if _level == 2  then
		if _value == 'Print' then
			dewdrop:AddLine(
				'text', 'Raid',
				'func', function() FuriousRaid.ExecuteCommand('print', s, 'RAID', FuriousUIState.listcomparator) end
			)
			dewdrop:AddLine(
				'text', 'Guild',
				'func', function() FuriousRaid.ExecuteCommand('print', s, 'GUILD', FuriousUIState.listcomparator) end
			)
			dewdrop:AddLine(
				'text', 'Officer',
				'func', function() FuriousRaid.ExecuteCommand('print', s, 'OFFICER', FuriousUIState.listcomparator) end
			)
			dewdrop:AddLine(
				'text',	'',
				'notClickable', true
			)
			local id, name
			id = 0
			for i=1, 10 do
				id, name = GetChannelName(i)
				if id ~= 0 then
					dewdrop:AddLine(
						'arg1', id,
						'text', name,
						'func', function(a) FuriousRaid.ExecuteCommand('print', s, 'CHANNEL', FuriousUIState.listcomparator, a) end
					)
				end
			end
			dewdrop:AddLine(
				'text',	'',
				'notClickable', true
			)
			dewdrop:AddLine(
				'text', 'Whisper',
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxText', '',
				'editBoxFunc', function(a) FuriousRaid.ExecuteCommand('print', s, 'WHISPER', FuriousUIState.listcomparator, a) end,
				'editBoxValidateFunc', function(a) return true end
			)
		end

		return
	end

	dewdrop:AddLine(
		'text', 'Player: |cffaaffaa' .. s .. '|r',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Timer',
		'checked', FuriousRaid.ExecuteCommand('timeractive', s),
		'func', function() FuriousRaid.ExecuteCommand('timer', s) end
	)
	dewdrop:AddLine(
		'text', 'Inform',
		'checked', FuriousRaidDB.sets[s].config.add.inform,
		'func', function() FuriousRaidDB.sets[s].config.add.inform = not FuriousRaidDB.sets[s].config.add.inform end
	)
	dewdrop:AddLine(
		'text', 'Broadcast',
		'checked', FuriousRaidDB.sets[s].config.add.broadcast,
		'func', function() FuriousRaidDB.sets[s].config.add.broadcast = not FuriousRaidDB.sets[s].config.add.broadcast end
	)

	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Add: Raid',
		'hasArrow', true,
		'hasEditBox', true,
		'editBoxText', '',
		'editBoxFunc', function(a)
			local r = FuriousRaid.GetRaidList()
			for k, v in pairs(r) do
				FuriousRaid.ExecuteCommand('add', s, k, tonumber(a))
			end
		end,
		'editBoxValidateFunc', function(a) return (tonumber(a) ~= nil) end
	)
	dewdrop:AddLine(
		'text', 'Add: Waitlist',
		'hasArrow', true,
		'hasEditBox', true,
		'editBoxText', '',
		'editBoxFunc', function(a)
			local r = FuriousRaid.GetWaitList()
			for k, v in pairs(r) do
				FuriousRaid.ExecuteCommand('add', s, k, tonumber(a))
			end
		end,
		'editBoxValidateFunc', function(a) return (tonumber(a) ~= nil) end
	)

	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Print',
		'value', 'Print',
		'hasArrow', true
	)
 	dewdrop:AddLine(
		'text', 'Configuration',
		'func', function() FuriousRaid.ExecuteCommand('showconfig', s) dewdrop:Close() end
	)

	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
 	dewdrop:AddLine(
		'text', 'Remove!',
		'func', function()
			FuriousRaid.RemoveSet(s)
			FuriousRaid.state.selected.set = nil

			FuriousUI.UpdateSet()
			FuriousUI.ClearList()
			FuriousUI.ClearSetDetail()

			dewdrop:Close()
		end
	)
end

local printwaitlist = function (_channel, _destination)
	comm:SendChatMessage('BULK', FuriousRaid.prefix, 'Waitlist', _channel, nil, _destination)
	comm:SendChatMessage('BULK', FuriousRaid.prefix, '===============================', _channel, nil, _destination)
	for k, v in pairs(FuriousRaidDB.waitlist) do
		comm:SendChatMessage('BULK', FuriousRaid.prefix, k, _channel, nil, _destination)
	end
	comm:SendChatMessage('BULK', FuriousRaid.prefix, '===============================', _channel, nil, _destination)
end
furiousUIL.menuNormalWaitlistButton = function (_level, _value)
	local p = FuriousRaid.state.selected.waitlist

	if _level == 2  then
		if _value == 'Print' then
			dewdrop:AddLine(
				'text', 'Raid',
				'func', function() printwaitlist('RAID') end
			)
			dewdrop:AddLine(
				'text', 'Guild',
				'func', function() printwaitlist('GUILD') end
			)
			dewdrop:AddLine(
				'text', 'Officer',
				'func', function() printwaitlist('OFFICER') end
			)
			dewdrop:AddLine(
				'text',	'',
				'notClickable', true
			)
			local id, name
			id = 0
			for i=1, 10 do
				id, name = GetChannelName(i)
				if id ~= 0 then
					dewdrop:AddLine(
						'arg1', id,
						'text', name,
						'func', function(a) printwaitlist('CHANNEL', a) end
					)
				end
			end
			dewdrop:AddLine(
				'text',	'',
				'notClickable', true
			)
			dewdrop:AddLine(
				'text', 'Whisper',
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxText', '',
				'editBoxFunc', function(a) printwaitlist('WHISPER', a) end,
				'editBoxValidateFunc', function(a) return true end
			)
		end

		return
	end

	dewdrop:AddLine(
		'text', 'Player: |cffaaffaa' .. p .. '|r',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Invite',
		'func', function() InviteUnit(p) FuriousRaidDB.waitlist[p] = nil FuriousUI.UpdateWaitlist() dewdrop:Close() end
	)
	dewdrop:AddLine(
		'text', 'Remove',
		'func', function() FuriousRaidDB.waitlist[p] = nil FuriousUI.UpdateWaitlist() dewdrop:Close() end
	)
	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Print',
		'value', 'Print',
		'hasArrow', true
	)
end


furiousUIL.menuFRaidButton = function (level ,value)
	dewdrop:AddLine(
		'text', 'Accept Broadcast',
		'checked', FuriousRaidDB.acceptbroadcast,
		'func', function() FuriousRaidDB.acceptbroadcast = not FuriousRaidDB.acceptbroadcast end
	)
	dewdrop:AddLine(
		'text', '',
		'notClickable', true
	)
	dewdrop:AddLine(
		'text', 'Clean Database',
		'func', function() FuriousRaid.DataIntegrityCheck() end
	)
	dewdrop:AddLine(
		'text', 'Transmit Database',
		'hasArrow', true,
		'hasEditBox', true,
		'editBoxText', '',
		'editBoxFunc', function(a) FuriousRaid.CommSendSync(a) end,
		'editBoxValidateFunc', function(a) return true end
	)
end


furiousUIL.onClickShowAddSetButton = function ()
	FuriousUIFrames.frames.filtereditbox:ClearFocus()
	FuriousUIDialogs.groups.setnamebox:SetFocus()

	FuriousUIDialogs.frames.main:Show()
end

furiousUIL.onClickAddSetButton = function ()
	local n = FuriousUIDialogs.groups.setnamebox:GetText()
	local c = UIDropDownMenu_GetText(FuriousUIDialogs.frames.dropdown)

	if n and FuriousRaid.state.sets[c] then
		FuriousRaid.AddSet(n, c)
		FuriousUI.UpdateSet()

		FuriousUIDialogs.frames.main:Hide()
	else
		print(FuriousRaid.prefixprint, 'error adding set')
	end
end

furiousUIL.onEnterPressedFilterEditbox = function (f)
	FuriousRaid.SetFilterFromString(f:GetText())

	FuriousUI.UpdateList()

	f:ClearFocus()
end

furiousUIL.onClickEditbox = function (f)
	f:SetFocus()
	f:HighlightText()
end



--
--
-- button templates

-- set and list buttons
furiousUIL.setSetListButtonText = function (self, a, b, c)
	self.fTextA:SetText(a)
	self.fTextB:SetText(b)
	self.fTextC:SetText(c)
end


furiousUIL.setSetListButtonColor = function (self, r, g, b)
	self.fTextA:SetTextColor(r, g, b)
	self.fTextB:SetTextColor(r, g, b)
	self.fTextC:SetTextColor(r, g, b)
end

furiousUIL.createSetListHeader = function (p, w, h, c)
	local f = CreateFrame('Button', nil, p)
	f.fID = 'HeaderA'
	f.fDirection = true
	f.fComparator = comparealpha
	f.fComparator = function (f)
		f.fDirection = not f.fDirection
		if f.fDirection then
			return comparealpha
		else
			return comparealphainverse
		end
	end
	f:SetHeight(h)
	f:SetWidth(w * 0.45)
	f:SetScript('OnClick', c)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetAllPoints(f)
	f.fTextA:SetFontObject(GameFontHighlight)
	f.fTextA:SetJustifyH('LEFT')

	local x = CreateFrame('Button', nil, p)
	x.fID = 'HeaderB'
	x.fDirection = true
	x.fComparator = function (f)
		f.fDirection = not f.fDirection
		if f.fDirection then
			return compareplayerclass
		else
			return compareplayerclassinverse
		end
	end
	x:SetHeight(h)
	x:SetWidth(w * 0.3)
	x:SetScript('OnClick', c)
	x:SetPoint('TOPLEFT', f, 'TOPRIGHT', 0, 0)

	f.fTextB = x:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetAllPoints(x)
	f.fTextB:SetFontObject(GameFontHighlight)
	f.fTextB:SetJustifyH('LEFT')

	local z = CreateFrame('Button', nil, p)
	z.fID = 'HeaderC'
	z.fDirection = true
	z.fComparator = function (f)
		f.fDirection = not f.fDirection
		if f.fDirection then
			return compareplayervalue
		else
			return compareplayervalueinverse
		end
	end
	z:SetHeight(h)
	z:SetWidth(w * 0.25)
	z:SetScript('OnClick', c)
	z:SetPoint('TOPLEFT', x, 'TOPRIGHT', 0, 0)

	f.fTextC = f:CreateFontString(nil, 'OVERLAY')
	f.fTextC:SetAllPoints(z)
	f.fTextC:SetFontObject(GameFontHighlight)
	f.fTextC:SetJustifyH('LEFT')

	f.SetMultiText = furiousUIL.setSetListButtonText
	f.SetMultiColor = furiousUIL.setSetListButtonColor

	f:Show()

	return f
end

furiousUIL.createSetListButton = function (p, w, h, c)
	local f = CreateFrame('Button', nil, p)
	f:SetHeight(h)
	f:SetWidth(w)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetHeight(h)
	f.fTextA:SetWidth(w * 0.45)
	f.fTextA:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
	f.fTextA:SetFontObject(GameFontNormal)
	f.fTextA:SetJustifyH('LEFT')

	f.fTextB = f:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetHeight(h)
	f.fTextB:SetWidth(w * 0.3)
	f.fTextB:SetPoint('TOPLEFT', f.fTextA, 'TOPRIGHT', 0, 0)
	f.fTextB:SetFontObject(GameFontNormal)
	f.fTextB:SetJustifyH('LEFT')

	f.fTextC = f:CreateFontString(nil, 'OVERLAY')
	f.fTextC:SetHeight(h)
	f.fTextC:SetPoint('TOPLEFT', f.fTextB, 'TOPRIGHT', 0, 0)
	f.fTextC:SetPoint('TOPRIGHT', f, 'TOPRIGHT', 0, 0)
	f.fTextC:SetFontObject(GameFontNormal)
	f.fTextC:SetJustifyH('LEFT')

	f.SetMultiText = furiousUIL.setSetListButtonText
	f.SetMultiColor = furiousUIL.setSetListButtonColor

	f:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
	f:SetScript('OnClick', c)

	f:SetBackdrop( {
			bgFile = 'Interface/Tooltips/UI-Tooltip-Background',
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		}
	)
	f:SetBackdropColor(0.5, 0.5, 0.75, 0)

	f:Show()

	return f
end



-- waitlist buttons
furiousUIL.setWaitlistButtonText = function (f, a, b, c)
	f.fTextA:SetText(a)
	f.fTextB:SetText(b)
end

furiousUIL.setWaitlistButtonColor = function (f, r, g, b)
	f.fTextA:SetTextColor(r, g, b)
	f.fTextB:SetTextColor(r, g, b)
end


furiousUIL.createWaitlistHeader = function (p, w, h, c)
	local f = CreateFrame('Button', nil, p)
	f.fID = 'HeaderA'
	f:SetHeight(h)
	f:SetWidth(w * 0.5)
	f:SetScript('OnClick', c)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetAllPoints(f)
	f.fTextA:SetFontObject(GameFontHighlight)
	f.fTextA:SetJustifyH('LEFT')

	local x = CreateFrame('Button', nil, p)
	x.fID = 'HeaderB'
	x:SetHeight(h)
	x:SetWidth(w * 0.5)
	x:SetScript('OnClick', c)
	x:SetPoint('TOPLEFT', f, 'TOPRIGHT', 0, 0)

	f.fTextB = x:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetAllPoints(x)
	f.fTextB:SetFontObject(GameFontHighlight)
	f.fTextB:SetJustifyH('LEFT')

	f.SetMultiText = furiousUIL.setWaitlistButtonText
	f.SetMultiColor = furiousUIL.setWaitlistButtonColor

	f:Show()

	return f
end

furiousUIL.createWaitlistButton = function (p, w, h, c)
	local f = CreateFrame('Button', nil, p)
	f:SetHeight(h)
	f:SetWidth(w)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetHeight(h)
	f.fTextA:SetWidth(w * 0.5)
	f.fTextA:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
	f.fTextA:SetFontObject(GameFontNormal)
	f.fTextA:SetJustifyH('LEFT')

	f.fTextB = f:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetHeight(h)
	f.fTextB:SetPoint('TOPLEFT', f.fTextA, 'TOPRIGHT', 0, 0)
	f.fTextB:SetPoint('TOPRIGHT', f, 'TOPRIGHT', 0, 0)
	f.fTextB:SetFontObject(GameFontNormal)
	f.fTextB:SetJustifyH('LEFT')

	f:Show()

	f.SetMultiText = furiousUIL.setWaitlistButtonText
	f.SetMultiColor = furiousUIL.setWaitlistButtonColor

	f:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
	f:SetScript('OnClick', c)

	f:SetBackdrop( {
			bgFile = 'Interface/Tooltips/UI-Tooltip-Background',
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		}
	)
	f:SetBackdropColor(0.5, 0.5, 0.75, 0)

	return f
end



-- detail buttons
furiousUIL.setItemHeaderText = function (f, a)
	f.fTextA:SetText(a)
end

furiousUIL.setItemHeaderColor = function (f, r, g, b)
	f.fTextA:SetTextColor(r, g, b)
end

furiousUIL.createItemHeader = function (p, w, h, c)
	local f = CreateFrame('Button', nil, p)
	f.fID = 'HeaderA'
	f:SetHeight(h)
	f:SetWidth(w)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetAllPoints(f)
	f.fTextA:SetFontObject(GameFontHighlight)
	f.fTextA:SetJustifyH('LEFT')

	f.SetMultiText = furiousUIL.setItemHeaderText
	f.SetMultiColor = furiousUIL.setItemHeaderColor

	f:Show()

	return f
end

furiousUIL.onEnterPlayerItemButton = function (f)
	if f and f.fLink then
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:SetHyperlink(f.fLink)
		GameTooltip:Show()
	end
end

furiousUIL.onLeavePlayerItemButton = function (f)
	if f and f.fLink then
		GameTooltip:Hide()
	end
end

furiousUIL.setPlayerItemButtonItem = function (f, item, when)
	f.fTextA:SetText(date('%Y-%m-%d  %H:%M', when))
	f.fTextB:SetText((item.set or 'UNKNOWN') .. ' for ' .. (item.value or 'UNKNOWN') .. ' hours ')
	f.fTextC:SetText(item.item.name)

	f.fIcon:SetTexture(GetItemIcon(tonumber(item.item.id)) or [[Interface\ICONS\INV_Misc_Questionmark]])

	f.fID = when
	f.fLink = item.item.link
end

furiousUIL.createPlayerItemButton = function (p, w, c)
	local f = CreateFrame('Button', nil, p)
	f:SetHeight(48)
	f:SetWidth(w)

	f.fTextC = f:CreateFontString(nil, 'OVERLAY')
	f.fTextC:SetHeight(16)
	f.fTextC:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
	f.fTextC:SetPoint('TOPRIGHT', f, 'TOPRIGHT', 0, 0)
	f.fTextC:SetFontObject(GameFontHighlight)
	f.fTextC:SetJustifyH('LEFT')

	f.fIcon = f:CreateTexture(nil, 'OVERLAY')
	f.fIcon:SetHeight(32)
	f.fIcon:SetWidth(32)
	f.fIcon:SetPoint('TOPLEFT', f.fTextC, 'BOTTOMLEFT', 0, 0)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetHeight(16)
	f.fTextA:SetPoint('TOPLEFT', f.fIcon, 'TOPRIGHT', 0, 0)
	f.fTextA:SetPoint('TOPRIGHT', f.fTextC, 'BOTTOMRIGHT', 0, 0)
	f.fTextA:SetFontObject(GameFontNormal)
	f.fTextA:SetJustifyH('LEFT')

	f.fTextB = f:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetHeight(16)
	f.fTextB:SetPoint('TOPLEFT', f.fTextA, 'BOTTOMLEFT', 0, 0)
	f.fTextB:SetPoint('BOTTOMRIGHT', f, 'BOTTOMRIGHT', 0, 0)
	f.fTextB:SetFontObject(GameFontNormal)
	f.fTextB:SetJustifyH('LEFT')

	f:Show()

	f.SetItem = furiousUIL.setPlayerItemButtonItem
	f:SetScript('OnEnter', furiousUIL.onEnterPlayerItemButton)
	f:SetScript('OnLeave', furiousUIL.onLeavePlayerItemButton)
	f:SetScript('OnClick', c)

	f:SetBackdrop( {
			bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		}
	)
	f:SetBackdropColor(0.5, 0.5, 0.75, 0.0)

	return f
end

furiousUIL.onEnterSetItemButton = function (f)
	if f and f.fLink then
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:SetHyperlink(f.fLink)
		GameTooltip:Show()
	end
end

furiousUIL.onLeaveSetItemButton = function (f)
	if f and f.fLink then
		GameTooltip:Hide()
	end
end

furiousUIL.setSetItemButtonItem = function (f, item, when)
	f.fTextA:SetText(date('%Y-%m-%d  %H:%M', when))
	f.fTextB:SetText('Awarded to ' .. (item.awarded or 'UNKNOWN'))
	f.fTextC:SetText(item.item.name)

	f.fIcon:SetTexture(GetItemIcon(tonumber(item.item.id)) or [[Interface\ICONS\INV_Misc_Questionmark]])

	f.fID = when
	f.fLink = item.item.link
end

furiousUIL.createSetItemButton = function (p, w, c)
	local f = CreateFrame('Button', nil, p)
	f:SetHeight(48)
	f:SetWidth(w)

	f.fTextC = f:CreateFontString(nil, 'OVERLAY')
	f.fTextC:SetHeight(16)
	f.fTextC:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
	f.fTextC:SetPoint('TOPRIGHT', f, 'TOPRIGHT', 0, 0)
	f.fTextC:SetFontObject(GameFontHighlight)
	f.fTextC:SetJustifyH('LEFT')

	f.fIcon = f:CreateTexture(nil, 'OVERLAY')
	f.fIcon:SetHeight(32)
	f.fIcon:SetWidth(32)
	f.fIcon:SetPoint('TOPLEFT', f.fTextC, 'BOTTOMLEFT', 0, 0)

	f.fTextA = f:CreateFontString(nil, 'OVERLAY')
	f.fTextA:SetHeight(16)
	f.fTextA:SetPoint('TOPLEFT', f.fIcon, 'TOPRIGHT', 0, 0)
	f.fTextA:SetPoint('TOPRIGHT', f.fTextC, 'BOTTOMRIGHT', 0, 0)
	f.fTextA:SetFontObject(GameFontNormal)
	f.fTextA:SetJustifyH('LEFT')

	f.fTextB = f:CreateFontString(nil, 'OVERLAY')
	f.fTextB:SetHeight(16)
	f.fTextB:SetPoint('TOPLEFT', f.fTextA, 'BOTTOMLEFT', 0, 0)
	f.fTextB:SetPoint('BOTTOMRIGHT', f, 'BOTTOMRIGHT', 0, 0)
	f.fTextB:SetFontObject(GameFontNormal)
	f.fTextB:SetJustifyH('LEFT')

	f:Show()

	f.SetItem = furiousUIL.setSetItemButtonItem
	f:SetScript('OnEnter', furiousUIL.onEnterSetItemButton)
	f:SetScript('OnLeave', furiousUIL.onLeaveSetItemButton)
	f:SetScript('OnClick', c)

	f:SetBackdrop( {
			bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		}
	)
	f:SetBackdropColor(0.5, 0.5, 0.75, 0.0)

	return f
end

-- dropdown
furiousUIL.onClickDetailDropdown = function (f)
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
	FuriousUIState.detail = this.value

	if this.value == 'Waitlist' then
		FuriousUIFrames.groups.waitlist.frames.main:Show()

		FuriousUIFrames.groups.setdetail.frames.main:Hide()
		FuriousUIFrames.groups.playerdetail.frames.main:Show()
	elseif this.value == 'Player Detail' then
		FuriousUIFrames.groups.playerdetail.frames.main:Show()

		FuriousUIFrames.groups.setdetail.frames.main:Hide()
		FuriousUIFrames.groups.waitlist.frames.main:Hide()
	else
		FuriousUIFrames.groups.setdetail.frames.main:Show()

		FuriousUIFrames.groups.playerdetail.frames.main:Hide()
		FuriousUIFrames.groups.waitlist.frames.main:Hide()
	end
end

furiousUIL.initializeDetailDropdown = function (f)
	local info = UIDropDownMenu_CreateInfo()

	info.text = 'Waitlist'
	info.value = 'Waitlist'
	info.owner = f
	info.checked = nil
	info.func = furiousUIL.onClickDetailDropdown
	UIDropDownMenu_AddButton(info)

	info.text = 'Player Detail'
	info.value = 'Player Detail'
	info.owner = f
	info.checked = nil
	info.func = furiousUIL.onClickDetailDropdown
	UIDropDownMenu_AddButton(info)

	info.text = 'Set Detail'
	info.value = 'Set Detail'
	info.owner = f
	info.checked = nil
	info.func = furiousUIL.onClickDetailDropdown
	UIDropDownMenu_AddButton(info)
end

--
furiousUIL.onClickAddSetDropdown = function (f)
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
 end

furiousUIL.initializeAddSetDropdown = function (f)
	local info = UIDropDownMenu_CreateInfo()

	-- hack for now
	if tablecount(FuriousRaid.state.sets) == 0 then
		info.text = 'Pro'
		info.value = 1
		info.owner = f
		info.checked = nil
		info.func = furiousUIL.onClickAddSetDropdown
		UIDropDownMenu_AddButton(info)
	end

	local i = 1
	for k, v in pairs(FuriousRaid.state.sets) do
		info.text = k
		info.value = i
		info.owner = f
		info.checked = nil
		info.func = furiousUIL.onClickAddSetDropdown
		UIDropDownMenu_AddButton(info)

		i = i + 1
	end
end



--
--
-- UI Frames.
local f
local backdrop = {
	bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 2,
		right = 2,
		top = 2,
		bottom = 2,
	},
}
local backdrop2 = {
	bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
	edgeFile = [[Interface\DialogFrame\UI-DialogBox-Border]],
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 4,
		right = 4,
		top = 4,
		bottom = 4,
	},
}



-- table of frames
FuriousUIFrames = { frames = { }, groups = { }, }
FuriousUIFrames.groups.set = { frames = { }, groups = { }, }
FuriousUIFrames.groups.list = { frames = { }, groups = { }, }
FuriousUIFrames.groups.setdetail = { frames = { }, groups = { }, }
FuriousUIFrames.groups.playerdetail = { frames = { }, groups = { }, }
FuriousUIFrames.groups.waitlist = { frames = { }, groups = { }, }

FuriousUIFrames.groups.set.groups.buttons = { frames = { }, groups = { }, }
FuriousUIFrames.groups.list.groups.buttons = { frames = { }, groups = { }, }
FuriousUIFrames.groups.playerdetail.groups.buttons = { frames = { }, groups = { }, }
FuriousUIFrames.groups.setdetail.groups.buttons = { frames = { }, groups = { }, }
FuriousUIFrames.groups.waitlist.groups.buttons = { frames = { }, groups = { }, }

FuriousUIDialogs = { frames = { }, groups = { }, }

FuriousUILDB = { frames = { }, groups = { }, }
FuriousUILDB.groups.buttons = { frames = { }, groups = { }, }



-- onupdate
local lastupdateset = 0
furiousUIL.onUpdateSet = function (f, e)
	lastupdateset = lastupdateset + e

	if (lastupdateset > 1) then
		lastupdateset = 0
		FuriousUI.UpdateSetTimers()
	end
end

local lastupdateLDB = 0
furiousUIL.onUpdateLDB = function (f, e)
	lastupdateLDB = lastupdateLDB + e

	if (lastupdateLDB > 1) then
		lastupdateLDB = 0

		for k, v in ipairs(FuriousUILDB.groups.buttons.frames) do
			if v.fIsSet then
				local t = FuriousRaidDB.sets[v.fTextA:GetText()].config.timer.remaining
				v.fTextB:SetText(timertostring(t))
			else
				break
			end
		end

		if not MouseIsOver(FuriousUILDB.frames.main) then
			FuriousUILDB.frames.main:Hide()
		end
	end
end



--
local fui = 0

-- main window
f = CreateFrame('Frame', 'FUIFrame' .. fui, UIParent)
f:SetBackdrop(backdrop2)
f:SetBackdropColor(0, 0, 0, 0.75)
f:SetPoint('CENTER', UIParent, 'CENTER', 0, 0)
f:SetWidth(700)
f:SetHeight(720)
f:EnableMouse(1)
f:SetMovable(1)
f:RegisterForDrag('LeftButton')
f:SetScript('OnDragStart', f.StartMoving)
f:SetScript('OnDragStop', f.StopMovingOrSizing)
FuriousUIFrames.frames.main = f
fui = fui + 1

-- set list window
f = CreateFrame('Frame', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('TOPLEFT', FuriousUIFrames.frames.main, 'TOPLEFT', 10, -55)
f:SetWidth(400)
f:SetHeight(150)
f:SetScript('OnUpdate', furiousUIL.onUpdateSet)
FuriousUIFrames.groups.set.frames.main = f
fui = fui + 1

-- player list window
f = CreateFrame('Frame', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.set.frames.main, 'BOTTOMLEFT', 0, -5)
f:SetPoint('BOTTOMLEFT', FuriousUIFrames.frames.main, 'BOTTOMLEFT', 10, 35)
f:SetWidth(400)
FuriousUIFrames.groups.list.frames.main = f
fui = fui + 1

-- player detail window
f = CreateFrame('Frame', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('TOPRIGHT', FuriousUIFrames.frames.main, 'TOPRIGHT', -10, -55)
f:SetPoint('BOTTOMLEFT', FuriousUIFrames.groups.list.frames.main, 'BOTTOMRIGHT', 5, 0)
FuriousUIFrames.groups.playerdetail.frames.main = f
FuriousUIFrames.groups.playerdetail.frames.main:Hide()
fui = fui + 1

-- set detail window
f = CreateFrame('Frame', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('TOPRIGHT', FuriousUIFrames.frames.main, 'TOPRIGHT', -10, -55)
f:SetPoint('BOTTOMLEFT', FuriousUIFrames.groups.list.frames.main, 'BOTTOMRIGHT', 5, 0)
FuriousUIFrames.groups.setdetail.frames.main = f
FuriousUIFrames.groups.setdetail.frames.main:Hide()
fui = fui + 1

-- waitlist window
f = CreateFrame('Frame', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('TOPRIGHT', FuriousUIFrames.frames.main, 'TOPRIGHT', -10, -55)
f:SetPoint('BOTTOMLEFT', FuriousUIFrames.groups.list.frames.main, 'BOTTOMRIGHT', 5, 0)
f:Hide()
FuriousUIFrames.groups.waitlist.frames.main = f
FuriousUIFrames.groups.waitlist.frames.main:Show()
fui = fui + 1

-- close button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIFrames.frames.main, 'UIPanelCloseButton')
f:SetPoint('TOPRIGHT', FuriousUIFrames.frames.main, 'TOPRIGHT', -2, -2)
FuriousUIFrames.frames.close = f
fui = fui + 1

-- fraid button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIFrames.frames.main)
f:SetPoint('TOPLEFT', FuriousUIFrames.frames.main, 'TOPLEFT', 14, -10)
f:SetWidth(200)
f:SetHeight(18)
f.fIcon = f:CreateTexture(nil, 'OVERLAY')
f.fIcon:SetWidth(18)
f.fIcon:SetHeight(18)
f.fIcon:SetTexture([[Interface\AddOns\FuriousRaid\icon]])
f.fIcon:SetPoint('TOPLEFT', f, 'TOPLEFT', 0, 0)
f.fText = f:CreateFontString(nil, 'OVERLAY')
f.fText:SetFontObject(GameFontHighlight)
f.fText:SetPoint('TOPLEFT', f.fIcon, 'TOPRIGHT', 2, 0)
f.fText:SetPoint('BOTTOMRIGHT', f, 'BOTTOMRIGHT', 0, 0)
f.fText:SetText('Furious|cff00ff00Raid|r')
f.fText:SetJustifyH('LEFT')
f.fText:SetJustifyV('MIDDLE')
f:EnableMouse(true)
f:SetScript('OnMouseDown', furiousUIL.onClickFRaidButton)
FuriousUIFrames.frames.fraid = f
fui = fui + 1


-- monitor button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIFrames.frames.main, 'UIPanelButtonTemplate')
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.list.frames.main, 'BOTTOMLEFT', 0, 0)
f:SetText('Monitor: OFF')
f:SetWidth(120)
f:SetHeight(22)
f:SetScript('OnClick', furiousUIL.onClickMonitorButton)
FuriousUIFrames.frames.monitor = f
fui = fui + 1

-- clear Waitlist button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIFrames.frames.main, 'UIPanelButtonTemplate')
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.waitlist.frames.main, 'BOTTOMLEFT', 0, 0)
f:SetText('Clear Waitlist')
f:SetWidth(120)
f:SetHeight(22)
f:SetScript('OnClick', furiousUIL.onClickClearWaitlistButton)
FuriousUIFrames.frames.clearwl = f
fui = fui + 1

-- addset button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIFrames.frames.main, 'UIPanelButtonTemplate')
f:SetPoint('BOTTOMLEFT', FuriousUIFrames.groups.set.frames.main, 'TOPLEFT', 0, 2)
f:SetText('Add Set')
f:SetWidth(120)
f:SetHeight(22)
f:SetScript('OnClick', furiousUIL.onClickShowAddSetButton)
FuriousUIFrames.frames.showaddset = f
fui = fui + 1

f = FuriousUIFrames.frames.main:CreateFontString(nil, 'OVERLAY')
f:SetPoint('TOPLEFT', FuriousUIFrames.frames.showaddset, 'TOPRIGHT', 12, 0)
f:SetHeight(20)
f:SetFontObject(GameFontNormal)
f:SetJustifyH('RIGHT')
f:SetText('Filter:')
FuriousUIFrames.frames.filterlabel = f
fui = fui + 1

f = CreateFrame('EditBox', 'FUI' .. fui, FuriousUIFrames.frames.main, 'InputBoxTemplate')
f:SetPoint('TOPLEFT', FuriousUIFrames.frames.filterlabel, 'TOPRIGHT', 10, -2)
f:SetHeight(20)
f:SetWidth(219)
f:SetAutoFocus(false)
f:ClearFocus()
f:SetScript('OnMouseDown', furiousUIL.onClickEditbox)
f:SetScript('OnEnterPressed', furiousUIL.onEnterPressedFilterEditbox)
FuriousUIFrames.frames.filtereditbox = f
fui = fui + 1


-- set buttons
f = furiousUIL.createSetListHeader(FuriousUIFrames.groups.set.frames.main, 388, 18, furiousUIL.onClickHeaderSetButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.set.frames.main, 'TOPLEFT', 6, 0)
f:SetMultiText('Set', 'Type', 'Timer')
FuriousUIFrames.groups.set.groups.buttons.frames['header'] = f
fui = fui + 1

f = furiousUIL.createSetListButton(FuriousUIFrames.groups.set.frames.main, 388, 18, furiousUIL.onClickNormalSetButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.set.groups.buttons.frames.header, 'BOTTOMLEFT', 0, 0)
table.insert(FuriousUIFrames.groups.set.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 7 do
	f = furiousUIL.createSetListButton(FuriousUIFrames.groups.set.frames.main, 388, 18, furiousUIL.onClickNormalSetButton)
	f:SetPoint('TOPLEFT', FuriousUIFrames.groups.set.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, 0)
	table.insert(FuriousUIFrames.groups.set.groups.buttons.frames, f)
	fui = fui + 1
end

-- set scroll frame
f = CreateFrame('ScrollFrame', 'FUISetScroll', FuriousUIFrames.groups.set.frames.main, 'FauxScrollFrameTemplate')
f:SetScript('OnVerticalScroll', function () FauxScrollFrame_OnVerticalScroll(FUISetScroll, arg1, 18, FuriousUI.UpdateSet) end)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.set.frames.main, 'TOPLEFT', 0, -8)
f:SetPoint('BOTTOMRIGHT', FuriousUIFrames.groups.set.frames.main, 'BOTTOMRIGHT', -30, 8)
f:SetWidth(20)
FuriousUIFrames.groups.set.frames.scroll = f
fui = fui + 1



-- list buttons
f = furiousUIL.createSetListHeader(FuriousUIFrames.groups.list.frames.main, 388, 18, furiousUIL.onClickHeaderListButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.list.frames.main, 'TOPLEFT', 6, 0)
f:SetMultiText('Player', 'Class', 'Value')
FuriousUIFrames.groups.list.groups.buttons.frames['header'] = f
fui = fui + 1

f = furiousUIL.createSetListButton(FuriousUIFrames.groups.list.frames.main, 388, 18, furiousUIL.onClickNormalListButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.list.groups.buttons.frames.header, 'BOTTOMLEFT', 0, 0)
table.insert(FuriousUIFrames.groups.list.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 25 do
	f = furiousUIL.createSetListButton(FuriousUIFrames.groups.list.frames.main, 388, 18, furiousUIL.onClickNormalListButton)
	f:SetPoint('TOPLEFT', FuriousUIFrames.groups.list.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, 0)
	table.insert(FuriousUIFrames.groups.list.groups.buttons.frames, f)
	fui = fui + 1
end

-- list scroll frame
f = CreateFrame('ScrollFrame', 'FUIListScroll', FuriousUIFrames.groups.list.frames.main, 'FauxScrollFrameTemplate')
f:SetScript('OnVerticalScroll', function () FauxScrollFrame_OnVerticalScroll(FUIListScroll, arg1, 18, FuriousUI.UpdateList) end)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.list.frames.main, 'TOPLEFT', 0, -8)
f:SetPoint('BOTTOMRIGHT', FuriousUIFrames.groups.list.frames.main, 'BOTTOMRIGHT', -30, 8)
f:SetWidth(20)
FuriousUIFrames.groups.list.frames.scroll = f
fui = fui + 1



-- waitlist buttons
f = furiousUIL.createWaitlistHeader(FuriousUIFrames.groups.waitlist.frames.main, 260, 18, furiousUIL.onClickHeaderWaitlistButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.waitlist.frames.main, 'TOPLEFT', 6, -8)
f:SetMultiText('Player', 'Class')
FuriousUIFrames.groups.waitlist.groups.buttons.frames['header'] = f
fui = fui + 1

f = furiousUIL.createWaitlistButton(FuriousUIFrames.groups.waitlist.frames.main, 260, 18, furiousUIL.onClickNormalWaitlistButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.waitlist.groups.buttons.frames.header, 'BOTTOMLEFT', 0, 0)
table.insert(FuriousUIFrames.groups.waitlist.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 30 do
	f = furiousUIL.createWaitlistButton(FuriousUIFrames.groups.waitlist.frames.main, 260, 18, furiousUIL.onClickNormalWaitlistButton)
	f:SetPoint('TOPLEFT', FuriousUIFrames.groups.waitlist.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, 0)
	table.insert(FuriousUIFrames.groups.waitlist.groups.buttons.frames, f)
	fui = fui + 1
end

-- waitlist scroll frame
f = CreateFrame('ScrollFrame', 'FUIWaitlistScroll', FuriousUIFrames.groups.waitlist.frames.main, 'FauxScrollFrameTemplate')
f:SetScript('OnVerticalScroll', function () FauxScrollFrame_OnVerticalScroll(FUIWaitlistScroll, arg1, 18, FuriousUI.UpdateWaitlist) end)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.waitlist.frames.main, 'TOPLEFT', 0, -8)
f:SetPoint('BOTTOMRIGHT', FuriousUIFrames.groups.waitlist.frames.main, 'BOTTOMRIGHT', -30, 8)
f:SetWidth(20)
FuriousUIFrames.groups.waitlist.frames.scroll = f
fui = fui + 1



-- player detail Item buttons
f = furiousUIL.createItemHeader(FuriousUIFrames.groups.playerdetail.frames.main, 260, 18)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.playerdetail.frames.main, 'TOPLEFT', 8, -8)
FuriousUIFrames.groups.playerdetail.groups.buttons.frames['header'] = f
fui = fui + 1

f = furiousUIL.createPlayerItemButton(FuriousUIFrames.groups.playerdetail.frames.main, 260, furiousUIL.onClickNormalPlayerDetailButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.playerdetail.groups.buttons.frames.header, 'BOTTOMLEFT', 0, -2)
table.insert(FuriousUIFrames.groups.playerdetail.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 11 do
	f = furiousUIL.createPlayerItemButton(FuriousUIFrames.groups.playerdetail.frames.main, 260, furiousUIL.onClickNormalPlayerDetailButton)
	f:SetPoint('TOPLEFT', FuriousUIFrames.groups.playerdetail.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, -6)
	table.insert(FuriousUIFrames.groups.playerdetail.groups.buttons.frames, f)
	fui = fui + 1
end

-- player detail scroll frame
f = CreateFrame('ScrollFrame', 'FUIPlayerDetailScroll', FuriousUIFrames.groups.playerdetail.frames.main, 'FauxScrollFrameTemplate')
f:SetScript('OnVerticalScroll', function () FauxScrollFrame_OnVerticalScroll(FUIPlayerDetailScroll, arg1, 54, FuriousUI.UpdatePlayerDetail) end)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.playerdetail.frames.main, 'TOPLEFT', 0, -8)
f:SetPoint('BOTTOMRIGHT', FuriousUIFrames.groups.playerdetail.frames.main, 'BOTTOMRIGHT', -30, 8)
f:SetWidth(20)
FuriousUIFrames.groups.playerdetail.frames.scroll = f
fui = fui + 1



-- set detail Item buttons
f = furiousUIL.createItemHeader(FuriousUIFrames.groups.setdetail.frames.main, 260, 18)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.setdetail.frames.main, 'TOPLEFT', 8, -8)
FuriousUIFrames.groups.setdetail.groups.buttons.frames['header'] = f
fui = fui + 1

f = furiousUIL.createSetItemButton(FuriousUIFrames.groups.setdetail.frames.main, 260, furiousUIL.onClickNormalSetDetailButton)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.setdetail.groups.buttons.frames.header, 'BOTTOMLEFT', 0, -2)
table.insert(FuriousUIFrames.groups.setdetail.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 11 do
	f = furiousUIL.createSetItemButton(FuriousUIFrames.groups.setdetail.frames.main, 260, furiousUIL.onClickNormalSetDetailButton)
	f:SetPoint('TOPLEFT', FuriousUIFrames.groups.setdetail.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, -6)
	table.insert(FuriousUIFrames.groups.setdetail.groups.buttons.frames, f)
	fui = fui + 1
end

-- set detail scroll frame
f = CreateFrame('ScrollFrame', 'FUISetDetailScroll', FuriousUIFrames.groups.setdetail.frames.main, 'FauxScrollFrameTemplate')
f:SetScript('OnVerticalScroll', function () FauxScrollFrame_OnVerticalScroll(FUISetDetailScroll, arg1, 54, FuriousUI.UpdateSetDetail) end)
f:SetPoint('TOPLEFT', FuriousUIFrames.groups.setdetail.frames.main, 'TOPLEFT', 0, -8)
f:SetPoint('BOTTOMRIGHT', FuriousUIFrames.groups.setdetail.frames.main, 'BOTTOMRIGHT', -30, 8)
f:SetWidth(20)
FuriousUIFrames.groups.setdetail.frames.scroll = f
fui = fui + 1



-- dropdown
f = CreateFrame('Frame', 'FUIDetailDropdown', FuriousUIFrames.frames.main, 'UIDropDownMenuTemplate')
f:SetPoint('TOPRIGHT', FuriousUIFrames.frames.main, 'TOPRIGHT', 5, -28)
UIDropDownMenu_SetWidth(f, 200)
UIDropDownMenu_Initialize(f, furiousUIL.initializeDetailDropdown)
UIDropDownMenu_SetSelectedValue(f, 'Waitlist')
FuriousUIState.detail = 'Waitlist'
FuriousUIFrames.frames.dropdown = f
fui = fui + 1



-- add set dialog
f = CreateFrame('Frame', 'FUI' .. fui, UIParent)
f:SetBackdrop(backdrop2)
f:SetBackdropColor(0, 0, 0, 0.5)
f:SetPoint('CENTER', UIParent, 'CENTER', 0, 0)
f:SetWidth(270)
f:SetHeight(150)
f:SetFrameStrata('HIGH')
f:EnableMouse(1)
f:SetMovable(1)
f:RegisterForDrag('LeftButton')
f:SetScript('OnDragStart', f.StartMoving)
f:SetScript('OnDragStop', f.StopMovingOrSizing)
f:Show()
FuriousUIDialogs.frames.main = f
fui = fui + 1

f = FuriousUIDialogs.frames.main:CreateFontString(nil, 'OVERLAY')
f:SetHeight(20)
f:SetWidth(80)
f:SetPoint('TOPLEFT', FuriousUIDialogs.frames.main, 'TOPLEFT', 5, -40)
f:SetFontObject(GameFontNormal)
f:SetJustifyH('RIGHT')
f:SetText('Set Type:')
FuriousUIDialogs.groups.settypelabel = f

f = CreateFrame('Frame', 'FUIAddSetDropdown', FuriousUIDialogs.frames.main, 'UIDropDownMenuTemplate')
f:SetPoint('TOPLEFT', FuriousUIDialogs.groups.settypelabel, 'TOPRIGHT', -10, 0)
UIDropDownMenu_SetWidth(f, 140)
UIDropDownMenu_Initialize(f, furiousUIL.initializeAddSetDropdown)
UIDropDownMenu_SetSelectedValue(f, 1)
FuriousUIDialogs.frames.dropdown = f
fui = fui + 1

f = FuriousUIDialogs.frames.main:CreateFontString(nil, 'OVERLAY')
f:SetHeight(20)
f:SetWidth(80)
f:SetPoint('TOPLEFT', FuriousUIDialogs.groups.settypelabel, 'BOTTOMLEFT', 0, -18)
f:SetFontObject(GameFontNormal)
f:SetJustifyH('RIGHT')
f:SetText('Set Name:')
FuriousUIDialogs.groups.setnamelabel = f
fui = fui + 1

f = CreateFrame('EditBox', 'FUI' .. fui, FuriousUIDialogs.frames.main, 'InputBoxTemplate')
f:SetPoint('TOPLEFT', FuriousUIDialogs.groups.setnamelabel, 'TOPRIGHT', 13, 0)
f:SetHeight(22)
f:SetWidth(146)
f:SetFrameStrata('DIALOG')
f:SetAutoFocus(false)
f:ClearFocus()
f:SetScript('OnMouseDown', furiousUIL.onClickEditbox)
FuriousUIDialogs.groups.setnamebox = f
fui = fui + 1

f = CreateFrame('Button', 'FUI' .. fui, FuriousUIDialogs.frames.main, 'UIPanelButtonTemplate')
f:SetPoint('TOPRIGHT', FuriousUIDialogs.groups.setnamebox, 'BOTTOMRIGHT', 0, -18)
f:SetText('Add Set')
f:SetWidth(120)
f:SetHeight(22)
f:SetScript('OnClick', furiousUIL.onClickAddSetButton)
FuriousUIDialogs.frames.addset = f
fui = fui + 1

-- close button
f = CreateFrame('Button', 'FUI' .. fui, FuriousUIDialogs.frames.main, 'UIPanelCloseButton')
f:SetPoint('TOPRIGHT', FuriousUIDialogs.frames.main, 'TOPRIGHT', -2, -2)
FuriousUIDialogs.frames.close = f
fui = fui + 1



--
-- LDB window
f = CreateFrame('Frame', 'FUI' .. fui, UIParent)
f:SetBackdrop(backdrop)
f:SetBackdropColor(0, 0, 0, 1)
f:SetBackdropBorderColor(1, 1, 1, 1)
f:SetPoint('CENTER', UIParent, 'CENTER', 0, 0)
f:SetWidth(240)
f:SetHeight(620)
f:SetScript('OnUpdate', furiousUIL.onUpdateLDB)
FuriousUILDB.frames.main = f
fui = fui + 1

f = furiousUIL.createWaitlistButton(FuriousUILDB.frames.main, 220, 16, furiousUIL.onClickNormalWaitlistButton)
f:SetPoint('TOPLEFT', FuriousUILDB.frames.main, 'TOPLEFT', 10, -10)
table.insert(FuriousUILDB.groups.buttons.frames, f)
fui = fui + 1

for i = 2, 30 do
	f = furiousUIL.createWaitlistButton(FuriousUILDB.frames.main, 220, 16, furiousUIL.onClickNormalWaitlistButton)
	f:SetPoint('TOPLEFT', FuriousUILDB.groups.buttons.frames[i-1], 'BOTTOMLEFT', 0, 0)
	table.insert(FuriousUILDB.groups.buttons.frames, f)
	fui = fui + 1
end


--
FuriousUIFrames.frames.main:Hide()
FuriousUIDialogs.frames.main:Hide()
FuriousUILDB.frames.main:Hide()



--
--
-- LDB
local ldb = LibStub:GetLibrary('LibDataBroker-1.1')

local getTipAnchor = function (f)
	local x, y = f:GetCenter()

	if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end

	local hhalf = (x > UIParent:GetWidth()*2/3) and 'RIGHT' or (x < UIParent:GetWidth()/3) and 'LEFT' or ''
	local vhalf = (y > UIParent:GetHeight()/2) and 'TOP' or 'BOTTOM'

	return vhalf..hhalf, f, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end

local frLDB = ldb:NewDataObject('Furious|cff00ff00Raid|r', {
	type = 'data source',
	text = 'Furious|cff00ff00Raid|r',
	icon = [[Interface\AddOns\FuriousRaid\icon]],
} )


frLDB.OnEnter = function (f)
	FuriousUILDB.frames.main:SetPoint(getTipAnchor(f))

	local p = { }
	for k, v in pairs(FuriousRaidDB.sets) do table.insert(p, k) end
	table.sort(p)

	i = 1
	for k, v in pairs(p) do
		local t = FuriousRaidDB.sets[v].config.timer.remaining
		FuriousUILDB.groups.buttons.frames[i].fIsSet = true
		FuriousUILDB.groups.buttons.frames[i]:SetMultiText(v, timertostring(t))
		FuriousUILDB.groups.buttons.frames[i]:SetScript('OnClick', nil)
		FuriousUILDB.groups.buttons.frames[i]:Show()
		i = i + 1
	end

	if tablecount(FuriousRaidDB.waitlist) > 0 then
		FuriousUILDB.groups.buttons.frames[i].fIsSet = nil
		FuriousUILDB.groups.buttons.frames[i]:SetMultiText('', '')
		FuriousUILDB.groups.buttons.frames[i]:SetScript('OnClick', nil)
		FuriousUILDB.groups.buttons.frames[i]:Show()
		i = i + 1

		FuriousUILDB.groups.buttons.frames[i]:SetMultiText('Waitlist', '')
		FuriousUILDB.groups.buttons.frames[i].fTextA:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		FuriousUILDB.groups.buttons.frames[i]:SetScript('OnClick', nil)
		FuriousUILDB.groups.buttons.frames[i]:Show()
		i = i + 1

		for k, v in pairs(FuriousRaidDB.waitlist) do
			if i > 30 then break end

			local class = 'UNKNOWN'
			if FuriousRaidDB.players[k] then
				class = FuriousRaidDB.players[k].class
				FuriousUILDB.groups.buttons.frames[i].fTextB:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b)
				FuriousUILDB.groups.buttons.frames[i].fTextA:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			else
				FuriousUILDB.groups.buttons.frames[i].fTextA:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
				FuriousUILDB.groups.buttons.frames[i].fTextB:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			end

			FuriousUILDB.groups.buttons.frames[i]:SetScript('OnClick', furiousUIL.onClickNormalWaitlistButton)
			FuriousUILDB.groups.buttons.frames[i]:SetMultiText(k, string.lower(class))
			FuriousUILDB.groups.buttons.frames[i]:Show()

			i = i + 1
		end
	end

	for j=i, 30, 1 do
		FuriousUILDB.groups.buttons.frames[i].fIsSet = nil
		FuriousUILDB.groups.buttons.frames[j]:Hide()
	end

	FuriousUILDB.frames.main:SetHeight((i-1)*16 + 20)
	FuriousUILDB.frames.main:Show()
end

frLDB.OnClick = function ()
	FuriousUI.Show()
end


--
--
-- slash command
SLASH_FRAID1 = '/fraid'
SLASH_FRAID2 = '/furiousraid'
SlashCmdList['FRAID'] = FuriousUI.Show
