--
--
-- ProSet

local comm = ChatThrottleLib
local acecomm = LibStub('AceComm-3.0')
local acetimer = LibStub('AceTimer-3.0')

local ProSet = {
	config = {
		version = 3.21,
		monitor = {
			autofilter = true,
			displaylabels = false,
		},
		timer = {
			awardwaitlist = true,
			looptime = 30 * 60,
			pointaward = 0.5,
			handle = nil,
			remaining = nil,
			showbar = nil,
		},
		value = {
			multiplier = {
				nonguild = 1,
				rank = { },
				whisper = { },
			},
			group = {
				nonguild = 10,
				rank = { },
				whisper = {
					pass = {
						absolute = -100,
						description = 'pass'
					},
					offspec = {
						relative = -2,
						description = 'offspec'
					},
				},
				description = { },
			},
		},
		add = {
			inform = true,
			broadcast = true,
		},
		clear = {
			broadcast = true,
		},
	},
	commands = { },
}

--
--
-- furious raid doesn't guarrante the player or the point set is there
local internalvalidate = function (_set, _player)
	FuriousRaid.AddPlayer(_player)

	if FuriousRaidDB.players[_player].sets[_set] == nil then
		FuriousRaidDB.players[_player].sets[_set] = 0
	end
end
local internalscore = function (_value, _multiplier, _group)
	return (_group * 1000) + (_value * _multiplier / 1000.0)
end
local internalvaluestring = function (_value, _multiplier, _group)
	local s = string.format('%5.1f', _value * _multiplier)
	if _multiplier ~= 1 then s = s .. ' (' .. _value .. ')' end

	if _group then
		s = s .. ' :' .. (FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.description[_group] or _group)
	end

	return s
end

ProSet.commands.add = function (_set, _player, value)
	internalvalidate(_set, _player)

	FuriousRaidDB.players[_player].sets[_set] = FuriousRaidDB.players[_player].sets[_set] + value

	if FuriousRaidDB.sets[_set].config.add.inform then
		local s = FuriousRaid.prefix .. ' ' .. value .. ' hours added to ' .. _set .. ', total is now ' .. ProSet.commands.playervaluestring(_set, _player)
		comm:SendChatMessage('BULK', FuriousRaid.prefix, s, 'WHISPER', nil, _player)
	end
end

ProSet.commands.clear = function (_set, _player)
	FuriousRaidDB.players[_player].sets[_set] = 0

	if FuriousRaidDB.sets[_set].config.add.inform then
		local s = FuriousRaid.prefix .. ' Your hours in ' .. _set .. ' have been reset to ' .. ProSet.commands.playervaluestring(_set, _player)
		comm:SendChatMessage('BULK', FuriousRaid.prefix, s, 'WHISPER', nil, _player)
	end
end

local internaltimer = function (_set)
	local r = FuriousRaid.GetRaidList()
	for k, v in pairs(r) do
		FuriousRaid.ExecuteCommand('add', _set, k, FuriousRaidDB.sets[_set].config.timer.pointaward)
	end

	if FuriousRaidDB.sets[_set].config.timer.awardwaitlist then
		local w = FuriousRaid.GetWaitList()
		for k, v in pairs(w) do
			FuriousRaid.ExecuteCommand('add', _set, k, FuriousRaidDB.sets[_set].config.timer.pointaward)
		end
	end

	if FuriousRaidDB.sets[_set].config.timer.showbar then
		FuriousRaid.StartBar(_set, FuriousRaidDB.sets[_set].config.timer.looptime, _set, [[Interface\ICONS\INV_Misc_PocketWatch_01]])
	end
end
ProSet.commands.timer = function (_set)
	if FuriousRaidDB.sets[_set].config.timer.handle then
		acetimer:CancelTimer(FuriousRaidDB.sets[_set].config.timer.handle)
		FuriousRaidDB.sets[_set].config.timer.handle = nil
		FuriousRaid.StopBar(_set)
	else
		FuriousRaidDB.sets[_set].config.timer.handle = acetimer:ScheduleRepeatingTimer(internaltimer, FuriousRaidDB.sets[_set].config.timer.looptime, _set)
		if FuriousRaidDB.sets[_set].config.timer.showbar then
			FuriousRaid.StartBar(_set, FuriousRaidDB.sets[_set].config.timer.looptime, _set, [[Interface\ICONS\INV_Misc_PocketWatch_01]])
		end
	end
end
ProSet.commands.timeractive = function (_set)
	if FuriousRaidDB.sets[_set].config.timer.handle then
		return acetimer:TimeLeft(FuriousRaidDB.sets[_set].config.timer.handle)
	end

	return false
end
local internaltimerrestart = function (_set)
	FuriousRaidDB.sets[_set].config.timer.handle = acetimer:ScheduleRepeatingTimer(internaltimer, FuriousRaidDB.sets[_set].config.timer.looptime, _set)
	internaltimer(_set)
end
ProSet.commands.timerrestart = function (_set, _time)
	FuriousRaidDB.sets[_set].config.timer.handle = acetimer:ScheduleTimer(internaltimerrestart, _time, _set)

	if FuriousRaidDB.sets[_set].config.timer.showbar then
		FuriousRaid.StartBar(_set, _time, _set, [[Interface\ICONS\INV_Misc_PocketWatch_01]])
	end
end

local monitorfirst = 'OPEN'
local monitorset
local monitors = {
	OPEN = {
		nextstate = 'CLOSED',
		monitortells = true,
		enableaward = false,
		buttontext = 'Monitor: Open',
	},
	CLOSED = {
		nextstate = 'STOPPED',
		monitortells = false,
		enableaward = true,
		buttontext = 'Monitor: Closed',
	},
	STOPPED = {
		nextstate = 'OPEN',
		monitortells = false,
		enableaward = false,
		buttontext = 'Monitor: Off',
	},
}
local monitorbids = {}
local internalsort = function (_a, _b, _set)
	local avalue = ProSet.commands.playervalue(_set, _a) * ProSet.commands.playervaluemultiplier(_set, _a)
	local bvalue = ProSet.commands.playervalue(_set, _b) * ProSet.commands.playervaluemultiplier(_set, _b)

	return avalue > bvalue
end
local internalsortbids = function (_a, _b)
	return _a.score > _b.score
end
ProSet.commands.monitor = function (_set, _monitor)
	local nextstate = monitorfirst
	if monitors[_monitor.stateid] then nextstate = monitors[_monitor.stateid].nextstate end

	if nextstate == monitorfirst then monitorbids = {} end

	_monitor.stateid = nextstate
	_monitor.monitortells = monitors[nextstate].monitortells
	_monitor.enableaward = monitors[nextstate].enableaward
	_monitor.buttontext = monitors[nextstate].buttontext

	if nextstate == 'OPEN' then
		local players = FuriousRaid.GetFilteredList()
		table.sort(players, function (a,b) return internalsort(a, b, _monitor.set) end)

		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Set: ' .. _monitor.set, 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Bidding open for: ' .. (_monitor.item.link or _monitor.item.name), 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '===============================', 'RAID')
		for k, v in ipairs(players) do
			comm:SendChatMessage('NORMAL', FuriousRaid.prefix, ProSet.commands.playervaluestring(_monitor.set, v) .. ' .. ' .. v .. ' (' .. string.lower(FuriousRaidDB.players[v].class) .. ')', 'RAID')
		end
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Bidding open for: ' .. (_monitor.item.link or _monitor.item.name), 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '===============================', 'RAID')
	elseif nextstate == 'CLOSED' then
		table.sort(monitorbids, internalsortbids)

		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Set: ' .. _monitor.set, 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Bidding closed for: ' .. (_monitor.item.link or _monitor.item.name), 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '===============================', 'RAID')
		if #monitorbids == 0 then
			comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '', 'RAID')
			comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'NO BIDS', 'RAID')
			comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '', 'RAID')
		else
			for k, v in ipairs(monitorbids) do
				comm:SendChatMessage('NORMAL', FuriousRaid.prefix,  v.valuestring .. ' .. ' .. v.name .. ((v.whisper and ' <' .. v.whisper .. '>') or '') , 'RAID')
			end
		end
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, 'Bidding closed for: ' .. (_monitor.item.link or _monitor.item.name), 'RAID')
		comm:SendChatMessage('NORMAL', FuriousRaid.prefix, '===============================', 'RAID')
	elseif nextstate == 'STOPPED' then
		_monitor.set = 'UNKNOWN'
		_monitor.when = 'UNKNOWN'
	end
end

local internalmonitorscore = function (_set, _player, _tell)
	local config = FuriousRaidDB.sets[_set].config

	if not _set or not _player then return 0 end
	internalvalidate(_set, _player)

	local whisper
	local value = FuriousRaidDB.players[_player].sets[_set]
	local multiplier = ProSet.commands.playervaluemultiplier(_set, _player)
	local group = ProSet.commands.playervaluegroup(_set, _player)
	local displaygroup = group

	if not FuriousRaidDB.sets[_set].config.monitor.displaylabels then displaygroup = nil end
	
	for k, v in pairs(config.value.multiplier.whisper) do
		if string.find(_tell, k) then
			if v.absolute then
				multiplier = v.absolute
				whisper = v.description
			end

			if v.relative then
				multiplier = multiplier + v.relative
				whisper = v.description
			end
		end
	end

	for k, v in pairs(config.value.group.whisper) do
		if string.find(_tell, k) then
			if v.absolute then
				group = v.absolute
				whisper = v.description
			end

			if v.relative then
				group = group + v.relative
				whisper = v.description
			end
		end
	end

	return internalscore(value, multiplier, group), whisper, internalvaluestring(value, multiplier, displaygroup)
end
ProSet.commands.monitortell = function (_set, _monitor, _player, _tell)
	local config = FuriousRaidDB.sets[_set].config

	-- remove player
	for k, v in pairs(monitorbids) do
		if v.name == _player then
			table.remove(monitorbids, k)
			break
		end
	end

	local s = 'bad things'
	if not FuriousRaid.IsFiltered(_player) then
		s = FuriousRaid.prefix .. ' Your bid for ' .. _monitor.item.link .. ' in ' .. _monitor.set .. ' has been rejected'
	elseif string.find(_tell, 'remove') then
		s = FuriousRaid.prefix .. ' Your bid for ' .. _monitor.item.link .. ' in ' .. _monitor.set .. ' has been removed'
	else
		local value = ProSet.commands.playervalue(_monitor.set, _player)
		local score, whisper, valuestring = internalmonitorscore(_monitor.set, _player, _tell)

		local bid = {}
		bid.tell = _tell
		bid.value = value
		bid.score = score
		bid.name = _player
		bid.whisper = whisper
		bid.valuestring = valuestring

		table.insert(monitorbids, bid)

		s = FuriousRaid.prefix .. ' Your ' .. ((bid.whisper and bid.whisper .. ' ') or '') .. 'bid for ' .. _monitor.item.link .. ' in ' .. _monitor.set .. ' has been accepted'
	end
	comm:SendChatMessage('BULK', FuriousRaid.prefix, s, 'WHISPER', nil, _player)
end

ProSet.commands.itemaward = function (_set, _player, _monitor)
	FuriousRaidDB.sets[_set].monitors[_monitor.when] = { }
	FuriousRaidDB.sets[_set].monitors[_monitor.when].awarded = _player
	FuriousRaidDB.sets[_set].monitors[_monitor.when].bids = monitorbids
	FuriousRaidDB.sets[_set].monitors[_monitor.when].item = {
		id = _monitor.item.id,
		name = _monitor.item.name,
		link = _monitor.item.link,
	}

	FuriousRaidDB.players[_player].items[_monitor.when] = { }
	FuriousRaidDB.players[_player].items[_monitor.when].set = _monitor.set
	FuriousRaidDB.players[_player].items[_monitor.when].item = {
		id = _monitor.item.id,
		name = _monitor.item.name,
		link = _monitor.item.link,
	}

	local value = ProSet.commands.playervalue(_set, _player)
	if value <= 0 then
		FuriousRaidDB.players[_player].items[_monitor.when].value = 1
		for k, v in pairs(FuriousRaidDB.sets[_set].monitors[_monitor.when].bids) do
			if v.name == _player then v.value = 1 break end
		end

		FuriousRaid.ExecuteCommand('add', _set, _player, -1)
	else
		FuriousRaidDB.players[_player].items[_monitor.when].value = value
		for k, v in pairs(FuriousRaidDB.sets[_set].monitors[_monitor.when].bids) do
			if v.name == _player then v.value = value break end
		end

		FuriousRaid.ExecuteCommand('clear', _set, _player)
	end

	ProSet.commands.monitor(_set, _monitor)
end

ProSet.commands.itembank = function (_set, _player, _monitor)
	FuriousRaidDB.sets[_set].monitors[_monitor.when] = { }
	FuriousRaidDB.sets[_set].monitors[_monitor.when].awarded = 'BANK'
	FuriousRaidDB.sets[_set].monitors[_monitor.when].bids = monitorbids
	FuriousRaidDB.sets[_set].monitors[_monitor.when].item = {
		id = _monitor.item.id,
		name = _monitor.item.name,
		link = _monitor.item.link,
	}

	ProSet.commands.monitor(_set, _monitor)
end

ProSet.commands.setvalue = function (_set)
	return ''
end

ProSet.commands.playervalue = function (_set, _player)
	if not _set or not _player then return 0 end
	internalvalidate(_set, _player)

	return FuriousRaidDB.players[_player].sets[_set]
end

ProSet.commands.playervaluemultiplier = function (_set, _player)
	local rankindex = FuriousRaid.state.guild.members[_player] and FuriousRaid.state.guild.members[_player].rankindex or false

	if not rankindex then return FuriousRaidDB.sets[_set].config.value.multiplier.nonguild end

	return FuriousRaidDB.sets[_set].config.value.multiplier.rank[rankindex] or 1
end

ProSet.commands.playervaluegroup = function (_set, _player)
	local rankindex = FuriousRaid.state.guild.members[_player] and FuriousRaid.state.guild.members[_player].rankindex or false

	if not rankindex then return FuriousRaidDB.sets[_set].config.value.group.nonguild end

	return FuriousRaidDB.sets[_set].config.value.group.rank[rankindex] or 10
end

ProSet.commands.playervaluestring = function (_set, _player, _showgroup)
	internalvalidate(_set, _player)

	local multiplier = ProSet.commands.playervaluemultiplier(_set, _player)
	local group = ProSet.commands.playervaluegroup(_set, _player)
	local value = FuriousRaidDB.players[_player].sets[_set]

	if not _showgroup then group = nil end

	return internalvaluestring(value, multiplier, group)
end

ProSet.commands.playerscore = function (_set, _player)
	if not _set or not _player then return 0 end
	internalvalidate(_set, _player)

	local value = FuriousRaidDB.players[_player].sets[_set]
	local multiplier = ProSet.commands.playervaluemultiplier(_set, _player)
	local group = ProSet.commands.playervaluegroup(_set, _player)
	return internalscore(value, multiplier, group)
end
ProSet.commands.print = function (_set, _channel, _comparator, _destination)
	local p = {}
	for k, v in pairs(FuriousRaidDB.players) do
		if FuriousRaid.IsFiltered(k) then
			table.insert(p, k)
		end
	end
	table.sort(p, _comparator)

	comm:SendChatMessage('BULK', FuriousRaid.prefix, 'Set: ' .. _set, _channel, nil, _destination)
	comm:SendChatMessage('BULK', FuriousRaid.prefix, '===============================', _channel, nil, _destination)
	for k, v in pairs(p) do
		comm:SendChatMessage('BULK', FuriousRaid.prefix, ProSet.commands.playervaluestring(_set, v) .. ' hours .. ' .. v .. ' (' .. string.lower(FuriousRaidDB.players[v].class) .. ')', _channel, nil, _destination)
	end
	comm:SendChatMessage('BULK', FuriousRaid.prefix, '===============================', _channel, nil, _destination)
end

--
--
-- config ui
local aceconfig = LibStub("AceConfig-3.0")
local aceconfigdialog = LibStub("AceConfigDialog-3.0")
local aceconfigregistry = LibStub("AceConfigRegistry-3.0")

--	config = {
--		version = 3.11,
--		timer = {
--			awardwaitlist = true,
--			looptime = 30 * 60,
--			pointaward = 0.5,
--			handle = nil,
--			remaining = nil,
--		},
--		add = {
--			inform = true,
--			broadcast = true,
--			multiplier = {
--				rank = { },
--				whisper = {
--					base = 1000,
--					specific = {
--						pass = 1,
--						alt = 1,
--					},
--				},
--			},
--		},
--	},

local prosetoptionsstate = {
	multiplierindex = 1,
	groupindex = 1,
	groupnumbers = { values = { 'wth', }, index = 1, },
}

local internalgroupnumbers = function (_set)
	local t
	local r

	t = { }
	t[10] = true -- default value
	t[FuriousRaidDB.sets[_set].config.value.group.nonguild] = true
	for k, v in pairs(FuriousRaidDB.sets[_set].config.value.group.rank) do
		t[v] = true
	end

	r = { }
	for k, v in pairs(FuriousRaidDB.sets[_set].config.value.group.whisper) do
		if v.relative then
			for kk, vv in pairs(t) do r[kk + v.relative] = true end
		end
	end
	for k, v in pairs(r) do t[k] = true end

	for k, v in pairs(FuriousRaidDB.sets[_set].config.value.group.whisper) do
		if v.absolute then t[v.absolute] = true end
	end

	r = { }
	for k, v in pairs(t) do table.insert(r, k) end
	table.sort(r)
	
	prosetoptionsstate.groupnumbers.values = { }
	for k, v in pairs(r) do prosetoptionsstate.groupnumbers.values[k] = tostring(v) end
	prosetoptionsstate.groupnumbers.index = 1
	
	aceconfigregistry:NotifyChange('ProSet')
end

local prosetoptions = {
	type = 'group',
	name = 'ProSet',
	args = {
		__name = {
			type = 'header',
			name = function () return FuriousRaid.state.selected.set end,
			order = 0,
		},
		__monitor = {
			type = 'group',
			name = 'Monitor',
			args = {
				__autofilter = {
					type = 'toggle',
					name = 'Autofilter',
					desc = 'Whether or not to automatically set the filter based on the item being looted.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.autofilter end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.autofilter = val end,
					width = 'full',
					order = 0,
				},
				__displaylabel = {
					type = 'toggle',
					name = 'Display Group Labels',
					desc = 'Whether or not group labels should be displayed when displaying the bidding results.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.displaylabels end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.monitor.displaylabels = val end,
					width = 'full',
					order = 1,
				},
			},
			inline = true,
			order = 1,
		},
		__timer = {
			type = 'group',
			name = 'Timer',
			args = {
				__awardwaitlist = {
					type = 'toggle',
					name = 'Award Waitlist',
					desc = 'Whether or not to award those on the waitlist when the timer event occurs.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.awardwaitlist end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.awardwaitlist = val end,
					width = 'full',
					order = 0,
				},
				__showbar = {
					type = 'toggle',
					name = 'Show Bar',
					desc = 'Whether or not to show a raidmod timer bar.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.showbar end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.showbar = val end,
					width = 'full',
					order = 1,
				},
				__looptime = {
					type = 'input',
					name = 'Event Internval',
					desc = 'Determines the repeat time for the timer event.',
					get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.looptime) end,
					set = function (info, val) if tonumber(val) ~= nil then FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.looptime = tonumber(val) end end,
					order = 2,
				},
				__pointaward = {
					type = 'input',
					name = 'Event Value',
					desc = 'Determines the amount to award those in the raid (and potentially the waitlist) when the timer event fires.',
					get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.pointaward) end,
					set = function (info, val) if tonumber(val) ~= nil then FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.timer.pointaward = tonumber(val) end end,
					order = 3,
				},
			},
			inline = true,
			order = 2,
		},
		__add = {
			name = 'Points',
			type = 'group',
			args = {
				__inform = {
					type = 'toggle',
					name = 'Inform',
					desc = 'Whether or not to whisper individuals when their points are modified.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.add.inform end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.add.inform = val end,
					order = 0,
				},
				__broadcast = {
					type = 'toggle',
					name = 'Broadcast',
					desc = 'Whether or not to broadcast point modifications to the addon channel.',
					get = function (info) return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.add.broadcast end,
					set = function (info,val) FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.add.broadcast = val end,
					order = 1,
				},
				__multipliers = {
					type = 'group',
					name = 'Multipliers',
					args = {
						__nonguild = {
							type = 'input',
							name = 'Non-guild',
							desc = 'Value to multiply the hours of a non-guildie by.',
							get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.multiplier.nonguild) end,
							set = function (info, val) if tonumber(val) ~= nil then FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.multiplier.nonguild = tonumber(val) end end,
							width = 'full',
							order = 0,
						},
						__rank = {
							type = 'select',
							name = 'Rank',
							values = FuriousRaid.state.guild.ranks,
							set = function (info,val) prosetoptionsstate.multiplierindex = val end,
							get = function (info) return prosetoptionsstate.multiplierindex end,
							order = 1,
						},
						__multiplier = {
							type = 'input',
							name = 'Multiplier',
							desc = 'Value to multiply the hours of members of the given guild rank by.',
							get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.multiplier.rank[prosetoptionsstate.multiplierindex - 1] or 1) end,
							set = function (info, val) if tonumber(val) ~= nil then FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.multiplier.rank[prosetoptionsstate.multiplierindex - 1] = tonumber(val) FuriousUI.UpdateList() end end,
							order = 2,
						},
					},
					inline = true,
					order = 2,
				},
				__groups = {
					type='group',
					name='Groups',
					args = {
						__nonguild = {
							type = 'input',
							name = 'Non-guild',
							desc = 'The group to put non-guildies in.',
							get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.nonguild) end,
							set = function (info, val) if tonumber(val) ~= nil then FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.nonguild = tonumber(val) end end,
							width = 'full',
							order = 0,
						},
						__rank = {
							type = 'select',
							name = 'Rank',
							values = FuriousRaid.state.guild.ranks,
							set = function (info,val) prosetoptionsstate.groupindex = val end,
							get = function (info) return prosetoptionsstate.groupindex end,
							order = 1,
						},
						__multiplier = {
							type = 'input',
							name = 'Group',
							desc = 'The group to put members of the given guild rank in.',
							get = function (info) return tostring(FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.rank[prosetoptionsstate.groupindex - 1] or 10) end,
							set = function (info, val)
								if tonumber(val) ~= nil then
									FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.rank[prosetoptionsstate.groupindex - 1] = tonumber(val)

									internalgroupnumbers(FuriousRaid.state.selected.set)
									FuriousUI.UpdateList()
								end
							end,
							order = 2,
						},
					},
					inline = true,
					order = 3,
				},
				__labels = {
					type = 'group',
					name = 'Group Labels',
					args = {
						__group = {
							type = 'select',
							name = 'Group',
							values = function () return prosetoptionsstate.groupnumbers.values end,
							set = function (info,val) prosetoptionsstate.groupnumbers.index = val end,
							get = function (info) return prosetoptionsstate.groupnumbers.index end,
							order = 1,
						},
						__description = {
							type = 'input',
							name = 'Label',
							get = function (info)
								local n = tonumber(prosetoptionsstate.groupnumbers.values[prosetoptionsstate.groupnumbers.index])
								return FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.description[n] or '<none>'
							end,
							set = function (info, val)
								local n = tonumber(prosetoptionsstate.groupnumbers.values[prosetoptionsstate.groupnumbers.index])
								FuriousRaidDB.sets[FuriousRaid.state.selected.set].config.value.group.description[n] = val
							end,
							order = 2,
						},
					},
					inline = true,
					order = 4,
				},
			},
			inline = true,
			order = 3,
		},
	},
}
aceconfig:RegisterOptionsTable('ProSet', prosetoptions)

ProSet.commands.showconfig = function (_set)
	internalgroupnumbers(_set)
	aceconfigdialog:Open('ProSet')
end

--
--
-- registration
FuriousRaid.RegisterSetCaste('Pro', ProSet)